<?php
/**
 * A NodeMessageRequestNodeListAnswer handler
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class NodeMessageRequestNodeListAnswerHandler extends BaseMessageHandler implements HandleableMessage, Registerable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set handler name
		$this->setHandlerName('message_request_node_list_answer');

		// Init message data array
		$this->messageDataElements = array(
			XmlRequestNodeListAnswerTemplateEngine::REQUEST_DATA_SESSION_ID,
			XmlRequestNodeListAnswerTemplateEngine::REQUEST_DATA_NODE_LIST,
			BaseXmlAnswerTemplateEngine::ANSWER_STATUS,
		);

		// Init message-data->configuration translation array
		$this->messageToConfig = array(
		/*
			XmlRequestNodeListTemplateEngine::REQUEST_DATA_SESSION_ID  => 'your_session_id'
		*/
		);

		// Init search data array
		$this->searchData = array(
			XmlRequestNodeListAnswerTemplateEngine::REQUEST_DATA_SESSION_ID,
		);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @return	$handlerInstance	An instance of a HandleableMessage class
	 */
	public final static function createNodeMessageRequestNodeListAnswerHandler () {
		// Get new instance
		$handlerInstance = new NodeMessageRequestNodeListAnswerHandler();

		// Return the prepared instance
		return $handlerInstance;
	}

	/**
	 * Handles data array of the message
	 *
	 * @param	$messageData		An array with message data to handle
	 * @param	$packageInstance	An instance of a Receivable class
	 * @return	void
	 * @throws	NoRequestNodeListAttemptedException	If this node has not attempted to announce itself
	 */
	public function handleMessageData (array $messageData, Receivable $packageInstance) {
		// Get node instance
		$nodeInstance = NodeObjectFactory::createNodeInstance();

		// Has this node attempted to announce itself?
		if (!$nodeInstance->ifNodeHasAnnouncementCompleted()) {
			/*
			 * This node has never announced itself, so it doesn't expect
			 * request-node-list answer messages.
			 */
			throw new NoRequestNodeListAttemptedException(array($this, $nodeInstance, $messageData), self::EXCEPTION_ANNOUNCEMENT_NOT_ATTEMPTED);
		} // END - if

		// Register the announcing node with this node
		$this->registerNodeByMessageData($messageData);

		// Handle the answer status element
		$nodeInstance->handleAnswerStatusByMessageData($messageData, $packageInstance);
	}

	/**
	 * Adds all required elements from given array into data set instance
	 *
	 * @param	$dataSetInstance	An instance of a StoreableCriteria class
	 * @param	$messageData		An array with all message data
	 * @return	void
	 */
	public function addArrayToDataSet (StoreableCriteria $dataSetInstance, array $messageData) {
		// Add generic first
		parent::addArrayToDataSet($dataSetInstance, $messageData);

		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('ANSWER-HANDLER[' . __METHOD__ . ':' . __LINE__ . ']: messageData=' . print_r($messageData, TRUE));

		// Add all ements
		foreach ($this->messageDataElements as $key) {
			// Debug message
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('ANSWER-HANDLER[' . __METHOD__ . ':' . __LINE__ . ']: messageData[' . $key . ']=' . $messageData[$key]);

			// Is it there?
			assert(isset($messageData[$key]));

			/*
			 * Add it, but remove any 'my-' prefixes as they are not used in
			 * database layer.
			 */
			$dataSetInstance->addCriteria(str_replace('my-', '', $key), $messageData[$key]);
		} // END - foreach
	}

	/**
	 * Initializes configuration data from given message data array
	 *
	 * @param	$messageData	An array with all message data
	 * @return	void
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	protected function initMessageConfigurationData (array $messageData) {
		// Please don't call this method
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

	/**
	 * Removes configuration data with given message data array from global
	 * configuration
	 *
	 * @param	$messageData	An array with all message data
	 * @return	void
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	protected function removeMessageConfigurationData (array $messageData) {
		// Please don't call this method
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}
}

// [EOF]
?>
